@@ -1,5 +1,5 @@
 //--------------------------------------------------------------------------
-// Program to pull the information out of various types of EXIF digital 
+// Program to pull the information out of various types of EXIF digital
 // camera files and show it in a reasonably consistent way
 //
 // This module handles basic Jpeg file handling
@@ -66,7 +66,7 @@ static void process_COM (const uchar * Data, int length)
     strcpy(ImageInfo.Comments,Comment);
 }
 
- 
+
 //--------------------------------------------------------------------------
 // Process a SOFn marker.  This is useful for the image dimensions
 //--------------------------------------------------------------------------
@@ -153,7 +153,7 @@ int ReadJpegSections (FILE * infile, ReadMode_t ReadMode)
         }
 
         Sections[SectionsRead].Type = marker;
-  
+
         // Read the length of the section.
         lh = fgetc(infile);
         ll = fgetc(infile);
@@ -176,6 +176,7 @@ int ReadJpegSections (FILE * infile, ReadMode_t ReadMode)
         if (Data == NULL){
             ErrFatal("Could not allocate memory");
         }
+        memset(Data, 0, 20);
         Sections[SectionsRead].Data = Data;
 
         // Store first two pre-read bytes.
@@ -190,7 +191,7 @@ int ReadJpegSections (FILE * infile, ReadMode_t ReadMode)
 
         switch(marker){
 
-            case M_SOS:   // stop before hitting compressed data 
+            case M_SOS:   // stop before hitting compressed data
                 // If reading entire image is requested, read the rest of the data.
                 if (ReadMode & READ_IMAGE){
                     int cp, ep, size;
@@ -225,7 +226,7 @@ int ReadJpegSections (FILE * infile, ReadMode_t ReadMode)
                 process_DQT(Data, itemlen);
                 break;
 
-            case M_DHT:   
+            case M_DHT:
                 // Use for jpeg quality guessing
                 process_DHT(Data, itemlen);
                 break;
@@ -247,7 +248,7 @@ int ReadJpegSections (FILE * infile, ReadMode_t ReadMode)
 
             case M_JFIF:
                 // Regular jpegs always have this tag, exif images have the exif
-                // marker instead, althogh ACDsee will write images with both markers.
+                // marker instead, although ACDsee will write images with both markers.
                 // this program will re-create this marker on absence of exif marker.
                 // hence no need to keep the copy from the file.
                 if (itemlen < 16){
@@ -286,7 +287,10 @@ int ReadJpegSections (FILE * infile, ReadMode_t ReadMode)
                 // There can be different section using the same marker.
                 if (ReadMode & READ_METADATA){
                     if (memcmp(Data+2, "Exif", 4) == 0){
-                        process_EXIF(Data, itemlen);
+                        if (!process_EXIF(Data, itemlen)){
+                            // malformatted exif sections, discard.
+                            free(Sections[--SectionsRead].Data);
+						}
                         break;
                     }else if (memcmp(Data+2, "http:", 5) == 0){
                         Sections[SectionsRead-1].Type = M_XMP; // Change tag for internal purposes.
@@ -299,7 +303,7 @@ int ReadJpegSections (FILE * infile, ReadMode_t ReadMode)
                         break;
                     }
                 }
-                // Oterwise, discard this section.
+                // Otherwise, discard this section.
                 free(Sections[--SectionsRead].Data);
                 break;
 
@@ -314,15 +318,15 @@ int ReadJpegSections (FILE * infile, ReadMode_t ReadMode)
                     free(Sections[--SectionsRead].Data);
                 }
                 break;
-           
-            case M_SOF0: 
-            case M_SOF1: 
-            case M_SOF2: 
-            case M_SOF3: 
-            case M_SOF5: 
-            case M_SOF6: 
-            case M_SOF7: 
-            case M_SOF9: 
+
+            case M_SOF0:
+            case M_SOF1:
+            case M_SOF2:
+            case M_SOF3:
+            case M_SOF5:
+            case M_SOF6:
+            case M_SOF7:
+            case M_SOF9:
             case M_SOF10:
             case M_SOF11:
             case M_SOF13:
@@ -451,7 +455,7 @@ int ReplaceThumbnail(const char * ThumbFileName)
         }
 
         // Adding or removing of thumbnail is not possible - that would require rearranging
-        // of the exif header, which is risky, and jhad doesn't know how to do.
+        // of the exif header, which is risky, and jhead doesn't know how to do.
         fprintf(stderr,"Image contains no thumbnail to replace - add is not possible\n");
         return FALSE;
     }
@@ -558,7 +562,7 @@ void DiscardAllButExif(void)
         CheckSectionsAllocated();
         Sections[SectionsRead++] = XmpKeeper;
     }
-}    
+}
 
 //--------------------------------------------------------------------------
 // Write image data back to disk.
@@ -580,22 +584,22 @@ void WriteJpegFile(const char * FileName)
     // Initial static jpeg marker.
     fputc(0xff,outfile);
     fputc(0xd8,outfile);
-    
+
     if (Sections[0].Type != M_EXIF && Sections[0].Type != M_JFIF){
         // The image must start with an exif or jfif marker.  If we threw those away, create one.
         static uchar JfifHead[18] = {
             0xff, M_JFIF,
-            0x00, 0x10, 'J' , 'F' , 'I' , 'F' , 0x00, 0x01, 
-            0x01, 0x01, 0x01, 0x2C, 0x01, 0x2C, 0x00, 0x00 
+            0x00, 0x10, 'J' , 'F' , 'I' , 'F' , 0x00, 0x01,
+            0x01, 0x01, 0x01, 0x2C, 0x01, 0x2C, 0x00, 0x00
         };
 
         if (ImageInfo.ResolutionUnit == 2 || ImageInfo.ResolutionUnit == 3){
             // Use the exif resolution info to fill out the jfif header.
-            // Usually, for exif images, there's no jfif header, so if wediscard
+            // Usually, for exif images, there's no jfif header, so if we discard
             // the exif header, use info from the exif header for the jfif header.
-            
+
             ImageInfo.JfifHeader.ResolutionUnits = (char)(ImageInfo.ResolutionUnit-1);
-            // Jfif is 1 and 2, Exif is 2 and 3 for In and cm respecively
+            // Jfif is 1 and 2, Exif is 2 and 3 for In and cm respectively
             ImageInfo.JfifHeader.XDensity = (int)ImageInfo.xResolution;
             ImageInfo.JfifHeader.YDensity = (int)ImageInfo.yResolution;
         }
@@ -605,16 +609,16 @@ void WriteJpegFile(const char * FileName)
         JfifHead[13] = (uchar)ImageInfo.JfifHeader.XDensity;
         JfifHead[14] = (uchar)(ImageInfo.JfifHeader.YDensity >> 8);
         JfifHead[15] = (uchar)ImageInfo.JfifHeader.YDensity;
-        
+
 
         fwrite(JfifHead, 18, 1, outfile);
 
         // use the values from the exif data for the jfif header, if we have found values
-        if (ImageInfo.ResolutionUnit != 0) { 
+        if (ImageInfo.ResolutionUnit != 0) {
             // JFIF.ResolutionUnit is {1,2}, EXIF.ResolutionUnit is {2,3}
-            JfifHead[11] = (uchar)ImageInfo.ResolutionUnit - 1; 
+            JfifHead[11] = (uchar)ImageInfo.ResolutionUnit - 1;
         }
-        if (ImageInfo.xResolution > 0.0 && ImageInfo.yResolution > 0.0) { 
+        if (ImageInfo.xResolution > 0.0 && ImageInfo.yResolution > 0.0) {
             JfifHead[12] = (uchar)((int)ImageInfo.xResolution>>8);
             JfifHead[13] = (uchar)((int)ImageInfo.xResolution);
 
@@ -633,7 +637,7 @@ void WriteJpegFile(const char * FileName)
 
     // Write the remaining image data.
     fwrite(Sections[a].Data, Sections[a].Size, 1, outfile);
-       
+
     fclose(outfile);
 }
 
@@ -666,7 +670,7 @@ int RemoveSectionType(int SectionType)
         if (Sections[a].Type == SectionType){
             // Free up this section
             free (Sections[a].Data);
-            // Move succeding sections back by one to close space in array.
+            // Move succeeding sections back by one to close space in array.
             memmove(Sections+a, Sections+a+1, sizeof(Section_t) * (SectionsRead-a-1));
             SectionsRead -= 1;
             a -= 1;
@@ -677,7 +681,7 @@ int RemoveSectionType(int SectionType)
 }
 
 //--------------------------------------------------------------------------
-// Remove sectons not part of image and not exif or comment sections.
+// Remove sections not part of image and not exif or comment sections.
 //--------------------------------------------------------------------------
 int RemoveUnknownSections(void)
 {
@@ -715,7 +719,7 @@ int RemoveUnknownSections(void)
             default:
                 // Unknown.  Delete.
                 free (Sections[a].Data);
-                // Move succeding sections back by one to close space in array.
+                // Move succeeding sections back by one to close space in array.
                 memmove(Sections+a, Sections+a+1, sizeof(Section_t) * (SectionsRead-a-1));
                 SectionsRead -= 1;
                 Modified = TRUE;
@@ -725,7 +729,7 @@ int RemoveUnknownSections(void)
 }
 
 //--------------------------------------------------------------------------
-// Add a section (assume it doesn't already exist) - used for 
+// Add a section (assume it doesn't already exist) - used for
 // adding comment sections and exif sections
 //--------------------------------------------------------------------------
 Section_t * CreateSection(int SectionType, unsigned char * Data, int Size)
@@ -736,7 +740,7 @@ Section_t * CreateSection(int SectionType, unsigned char * Data, int Size)
 
     NewIndex = 0; // Figure out where to put the comment section.
     if (SectionType == M_EXIF){
-        // Exif alwas goes first!
+        // Exif always goes first!
     }else{
         for (;NewIndex < 3;NewIndex++){ // Maximum fourth position (just for the heck of it)
             if (Sections[NewIndex].Type == M_JFIF) continue; // Put it after Jfif
@@ -751,7 +755,7 @@ Section_t * CreateSection(int SectionType, unsigned char * Data, int Size)
 
     CheckSectionsAllocated();
     for (a=SectionsRead;a>NewIndex;a--){
-        Sections[a] = Sections[a-1];          
+        Sections[a] = Sections[a-1];
     }
     SectionsRead += 1;
 
